package com.vctgo.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.core.constant.CacheConstants;
import com.vctgo.common.core.constant.UserConstants;
import com.vctgo.common.core.exception.ServiceException;
import com.vctgo.common.core.text.Convert;
import com.vctgo.common.core.utils.ServletUtils;
import com.vctgo.common.core.utils.SpringUtils;
import com.vctgo.common.core.utils.StrUtils;
import com.vctgo.common.datascope.annotation.DataScope;
import com.vctgo.common.mybatisplus.constant.MybatisPageConstants;
import com.vctgo.common.redis.service.RedisService;
import com.vctgo.common.security.utils.SecurityUtils;
import com.vctgo.system.api.domain.SysPlatformRole;
import com.vctgo.system.api.domain.SysPlatformUser;
import com.vctgo.system.api.model.LoginUser;
import com.vctgo.system.domain.SysPlatformRoleDept;
import com.vctgo.system.domain.SysPlatformRoleMenu;
import com.vctgo.system.domain.SysPlatformUserRole;
import com.vctgo.system.mapper.SysPlatformRoleDeptMapper;
import com.vctgo.system.mapper.SysPlatformRoleMapper;
import com.vctgo.system.mapper.SysPlatformRoleMenuMapper;
import com.vctgo.system.mapper.SysPlatformUserRoleMapper;
import com.vctgo.system.service.ISysPlatformRoleService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 角色 业务层处理
 *
 * @author vctgo
 */
@Service
public class SysPlatformRoleServiceImpl implements ISysPlatformRoleService
{
    @Autowired
    private SysPlatformRoleMapper roleMapper;

    @Autowired
    private SysPlatformRoleMenuMapper roleMenuMapper;

    @Autowired
    private SysPlatformUserRoleMapper userRoleMapper;

    @Autowired
    private SysPlatformRoleDeptMapper roleDeptMapper;

    @Autowired
    private RedisService redisService;


    /**
     * 根据条件分页查询角色数据
     *
     * @param role 角色信息
     * @return 角色数据集合信息
     */
    @Override
    @DataScope(deptAlias = "d")
    public IPage<SysPlatformRole> selectRolePage(SysPlatformRole role)
    {
        Page mpPage =new Page(Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_NUM),1L)
                ,Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_SIZE),10L));
        return roleMapper.selectRoleList(mpPage,role);
    }

    /**
     * 根据条件分页查询角色数据
     *
     * @param role 角色信息
     * @return 角色数据集合信息
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysPlatformRole> selectRoleList(SysPlatformRole role)
    {
        return roleMapper.selectRoleList(role);
    }

    /**
     * 根据用户ID查询角色
     *
     * @param userId 用户ID
     * @return 角色列表
     */
    @Override
    public List<SysPlatformRole> selectRolesByUserId(Long userId)
    {
        List<SysPlatformRole> userRoles = roleMapper.selectRolePermissionByUserId(userId);
        List<SysPlatformRole> roles = selectRoleAll();
        for (SysPlatformRole role : roles)
        {
            for (SysPlatformRole userRole : userRoles)
            {
                if (role.getRoleId().longValue() == userRole.getRoleId().longValue())
                {
                    role.setFlag(true);
                    break;
                }
            }
        }
        return roles;
    }

    /**
     * 根据用户ID查询权限
     *
     * @param userId 用户ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectRolePermissionByUserId(Long userId)
    {
        List<SysPlatformRole> perms = roleMapper.selectRolePermissionByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (SysPlatformRole perm : perms)
        {
            if (StrUtils.isNotNull(perm))
            {
                permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
            }
        }
        return permsSet;
    }

    /**
     * 查询所有角色
     *
     * @return 角色列表
     */
    @Override
    public List<SysPlatformRole> selectRoleAll()
    {
        return SpringUtils.getAopProxy(this).selectRoleList(new SysPlatformRole());
    }

    /**
     * 根据用户ID获取角色选择框列表
     *
     * @param userId 用户ID
     * @return 选中角色ID列表
     */
    @Override
    public List<Long> selectRoleListByUserId(Long userId)
    {
        return roleMapper.selectRoleListByUserId(userId);
    }

    /**
     * 通过角色ID查询角色
     *
     * @param roleId 角色ID
     * @return 角色对象信息
     */
    @Override
    public SysPlatformRole selectRoleById(Long roleId)
    {
        return roleMapper.selectRoleById(roleId);
    }


    /**
     * 通过角色Id获取对应的用户Id
     *
     * @return 结果
     */
    public List<Long> selectUserIdListByRoleId(Long roleId) {
        LambdaQueryWrapper<SysPlatformUserRole> wrapper = Wrappers.lambdaQuery();
        wrapper.select(SysPlatformUserRole::getUserId)
               .eq(SysPlatformUserRole::getRoleId, roleId);
        List<SysPlatformUserRole> sysUserRoleList = userRoleMapper.selectList(wrapper);
        List<Long> userIdList = sysUserRoleList.stream().map(SysPlatformUserRole::getUserId).collect(Collectors.toList());
        return CollectionUtils.isNotEmpty(userIdList) ? userIdList : Collections.emptyList();
    }

    /**
     * 校验角色名称是否唯一
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean checkRoleNameUnique(SysPlatformRole role)
    {
        Long roleId = StrUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
        SysPlatformRole info = roleMapper.checkRoleNameUnique(role.getRoleName());
        if (StrUtils.isNotNull(info) && info.getRoleId().longValue() != roleId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色权限是否唯一
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean checkRoleKeyUnique(SysPlatformRole role)
    {
        Long roleId = StrUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
        SysPlatformRole info = roleMapper.checkRoleKeyUnique(role.getRoleKey());
        if (StrUtils.isNotNull(info) && info.getRoleId().longValue() != roleId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色是否允许操作
     *
     * @param role 角色信息
     */
    @Override
    public void checkRoleAllowed(SysPlatformRole role)
    {
        if (StrUtils.isNotNull(role.getRoleId()) && role.isAdmin())
        {
            throw new ServiceException("不允许操作超级管理员角色");
        }
    }

    /**
     * 校验角色是否有数据权限
     *
     * @param roleId 角色id
     */
    @Override
    public void checkRoleDataScope(Long roleId)
    {
        if (!SysPlatformUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysPlatformRole role = new SysPlatformRole();
            role.setRoleId(roleId);
            List<SysPlatformRole> roles = SpringUtils.getAopProxy(this).selectRoleList(role);
            if (StrUtils.isEmpty(roles))
            {
                throw new ServiceException("没有权限访问角色数据！");
            }
        }
    }

    /**
     * 通过角色ID查询角色使用数量
     *
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    public int countUserRoleByRoleId(Long roleId)
    {
        return userRoleMapper.countUserRoleByRoleId(roleId);
    }

    /**
     * 新增保存角色信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertRole(SysPlatformRole role)
    {
        // 新增角色信息
        roleMapper.insertRole(role);
        return insertRoleMenu(role);
    }

    /**
     * 修改保存角色信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateRole(SysPlatformRole role)
    {
        // 修改角色信息
        roleMapper.updateRole(role);
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(role.getRoleId());
        int count = insertRoleMenu(role);

        // 获取使用该角色的用户id
        List<Long> userIdList = selectUserIdListByRoleId(role.getRoleId());
        // 凡是涉及到该角色的人员都推出登录
        List<String> keyList = new ArrayList<>();
        Collection<String> keys = redisService.keys(CacheConstants.LOGIN_TOKEN_KEY + "*");
        for (String key : keys) {
            LoginUser onlineUser = redisService.getCacheObject(key);
            if (Objects.nonNull(onlineUser) && userIdList.contains(onlineUser.getUserid())) {
                keyList.add(key);
            }
        }
        redisService.deleteObject(keyList);
        return count;
    }

    /**
     * 修改角色状态
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public int updateRoleStatus(SysPlatformRole role)
    {
        return roleMapper.updateRole(role);
    }

    /**
     * 修改数据权限信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int authDataScope(SysPlatformRole role)
    {
        // 修改角色信息
        roleMapper.updateRole(role);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(role.getRoleId());
        // 新增角色和部门信息（数据权限）
        return insertRoleDept(role);
    }

    /**
     * 新增角色菜单信息
     *
     * @param role 角色对象
     */
    public int insertRoleMenu(SysPlatformRole role)
    {
        int rows = 1;
        // 新增用户与角色管理
        List<SysPlatformRoleMenu> list = new ArrayList<SysPlatformRoleMenu>();
        for (Long menuId : role.getMenuIds())
        {
            SysPlatformRoleMenu rm = new SysPlatformRoleMenu();
            rm.setRoleId(role.getRoleId());
            rm.setMenuId(menuId);
            list.add(rm);
        }
        if (list.size() > 0)
        {
            rows = roleMenuMapper.batchRoleMenu(list);
        }
        return rows;
    }

    /**
     * 新增角色部门信息(数据权限)
     *
     * @param role 角色对象
     */
    public int insertRoleDept(SysPlatformRole role)
    {
        int rows = 1;
        // 新增角色与部门（数据权限）管理
        List<SysPlatformRoleDept> list = new ArrayList<>();
        for (Long deptId : role.getDeptIds())
        {
            SysPlatformRoleDept rd = new SysPlatformRoleDept();
            rd.setRoleId(role.getRoleId());
            rd.setDeptId(deptId);
            list.add(rd);
        }
        if (list.size() > 0)
        {
            rows = roleDeptMapper.batchRoleDept(list);
        }
        return rows;
    }

    /**
     * 通过角色ID删除角色
     *
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteRoleById(Long roleId)
    {
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(roleId);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(roleId);
        return roleMapper.deleteRoleById(roleId);
    }

    /**
     * 批量删除角色信息
     *
     * @param roleIds 需要删除的角色ID
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteRoleByIds(Long[] roleIds)
    {
        for (Long roleId : roleIds)
        {
            checkRoleAllowed(new SysPlatformRole(roleId));
            checkRoleDataScope(roleId);
            SysPlatformRole role = selectRoleById(roleId);
            if (countUserRoleByRoleId(roleId) > 0)
            {
                throw new ServiceException(String.format("%1$s已分配,不能删除", role.getRoleName()));
            }
        }
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenu(roleIds);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDept(roleIds);
        return roleMapper.deleteRoleByIds(roleIds);
    }

    /**
     * 取消授权用户角色
     *
     * @param userRole 用户和角色关联信息
     * @return 结果
     */
    @Override
    public int deleteAuthUser(SysPlatformUserRole userRole)
    {
        return userRoleMapper.deleteUserRoleInfo(userRole);
    }

    /**
     * 批量取消授权用户角色
     *
     * @param roleId 角色ID
     * @param userIds 需要取消授权的用户数据ID
     * @return 结果
     */
    @Override
    public int deleteAuthUsers(Long roleId, Long[] userIds)
    {
        return userRoleMapper.deleteUserRoleInfos(roleId, userIds);
    }

    /**
     * 批量选择授权用户角色
     *
     * @param roleId 角色ID
     * @param userIds 需要授权的用户数据ID
     * @return 结果
     */
    @Override
    public int insertAuthUsers(Long roleId, Long[] userIds)
    {
        // 新增用户与角色管理
        List<SysPlatformUserRole> list = new ArrayList<>();
        for (Long userId : userIds)
        {
            SysPlatformUserRole ur = new SysPlatformUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
        }
        return userRoleMapper.batchUserRole(list);
    }
}
